#include "io430.h"
#include "radio.c"
#pragma diag_suppress=Pa082
#pragma diag_suppress=Pe111

void Clock_config();				// function prototypes
void Ports_config();
void SPI_config();
void Timer0_config();
void WDT_config();
void CC1101_config();
void Send_Radio(const char*, char, char);
void Send_Strobe(char);
void Check_Packet();
void Beep();

char SPIbuf[16];
char packet[] = {6, 0xAA, 0xBB, 'M', 'A', 'I', 'L'}; // radio packet
const char* dataPtr;				// pointer to byte to send via SPI
char* dataRxPtr;
char dataCnt;						// # of remaining bytes to send
char dataRxCnt;
char recFlg;

#define CS	    	P2				// radio CS on P1.2
#define GDO0    	P3				// radio GDO0 signal at P1.3
#define GDO2    	P4				// radio GDO2 signal at P1.4
#define SCLK		P5				// SPI interface
#define MOSI		P6				// -- " --
#define MISO		P7				// -- " --
#define RST_BTN  	P1    			// Reset Button at P1.1
#define LED     	P7  			// LED on P2.7
#define BUZZ      	P6      		// BUZZER ON P2.6

int main()
{
	WDTCTL = WDTPW + WDTHOLD;		// disable Watchdog
	Ports_config();					// set up ports for GPIO
	Clock_config();					// set 8 MHz MSCK, 2 MHz SMCLK		
	SPI_config();					// configure USI module for SPI
	CC1101_config();				// init CC1101 radio
	Timer0_config();				// set up timer for 2.73 KHz PWM
	WDT_config();					// configure WDT for buzzer
 	
	while(1)						// main app loop
	{  
	  	Send_Strobe(SFRX);			// flush RX FIFO
		Send_Strobe(SRX);			// start RX mode
		P1IES_bit.GDO0 = 0;			// set INT on low-to-high transition
		P1IFG = 0;					// clear port IF
		P1IE = RST_BTN + GDO0;		// enable port pin interrupts
		__low_power_mode_3();		// wait for receiving the sync word
		if (!recFlg) continue;		// it was button INT
		
		P1IES_bit.GDO0 = 1;			// set INT on high-to-low transition
		P1IFG = 0;					// clear IF
		P1IE_bit.GDO0 = 1;			// enable GDO0 pin INT
		__low_power_mode_3();		// wait for receiving a packet in FIFO
  
		Check_Packet();				// validate receive packet in FIFO     	  
	}
	return 0;
}

void Check_Packet()
{
  	char packLen, index;
    Send_Radio(NUM_RXBYTES, 1, 1); 	// get received packet length   
    packLen = *dataRxPtr;
	if ((packLen == 0) || (packLen != (packet[0]+3)))
	  	return;						// ignore inappropriate length packet
    
    Send_Radio(RX_FIFO_BURST, 1, packLen); // load packet from RX FIFO
	packLen = *dataRxPtr;			// received data length
	dataRxPtr++;
	index = 1;      				// start of mail packet that we will compare
    while( packLen > 0)
    {
        if (*dataRxPtr != packet[index])   //does received match packet?
			break;  
		packLen--;                 	// decrement NumRec and move to next piece of data
		index++;               		// if yes increment index
		dataRxPtr++;   
	}
	if (packLen == 0) 				// were all parts matched?
 	{
 		P2OUT_bit.LED = 1;			// power on LED (if off)
     	Beep();
	}
}  

void Beep()
{
  	char cycles = 8;       			//4 on-off cycles for beep
  	TA0CTL_bit.TACLR = 1;			// clear timer counter
  	WDTCTL = WDTPW + WDTTMSEL + WDTCNTCL + WDTSSEL
		  + WDTIS1;					// start WDT
  	while (cycles > 0)
  	{
     	__low_power_mode_0();		// Use watchdog timer to hold for ~.75 sec between each cycle
 		TA0CTL ^= MC0;				// toggle the buzzer
		cycles--;
  	}
  	WDTCTL = WDTPW + WDTTMSEL + WDTCNTCL + WDTSSEL
		  + WDTIS1 + WDTHOLD;		// stop WDT
}

void Send_Strobe(char s)			// send strobe s to the radio
{
	Send_Radio(&s, 1, 0);  
}

void Send_Radio(const char* a, char l, char rxL) 
{									// send array a of bytes via SPI; l is its length
  	dataCnt = l;
	dataRxCnt = rxL;
	if (rxL)
	{
	 	dataRxPtr = SPIbuf;
		dataCnt++;
	}	  
	dataPtr = a;
  	P1OUT_bit.CS = 0;				// CS down
	while (P1IN_bit.GDO2){}			// wait for radio interface ready
	USICTL1_bit.USIIE = 1;			// enable USI counter interrupt 
	__low_power_mode_0();			// Enter LPM0, interrupts enabled
	P1OUT_bit.CS = 1;				// CS up
	dataRxPtr -= rxL;
}  

void CC1101_config()
{
	TA0CTL = TASSEL0 + TACLR + TAIE;// ACLK, enable interrupt
	TA0CCR0 = 1200;					// ~100ms power-up delay
  	TA0CTL_bit.MC0 = 1;				// start timer in Up mode
	__low_power_mode_0();			// wait for timer to expire
	TA0CTL_bit.MC0 = 0;				// stop timer

	Send_Strobe(SRES);				// reset radio
  	dataCnt = 48;					// send that many bytes
	dataRxCnt = 0;					// # of bytes to receive
	dataPtr = cc1101_regs;			// config bytes
	P1OUT_bit.CS = 0;				// CS down
	while (P1IN_bit.MISO){}			// wait for radio interface ready
	USICTL1_bit.USIIE = 1;			// enable USI counter interrupt 
	__low_power_mode_0();			// Enter LPM0, interrupts enabled
	P1OUT_bit.CS = 1;				// CS up 
}

void Timer0_config()				// Timer0 setup for short delays
{
	TA0CTL = TASSEL1 + TACLR;		// SMCLK of 2 MHz
	TACCR0 = 732;					// generate 2.73 KHz
	TACCR1 = 366;					// 50% duty cycle
	TACCTL1 = OUTMOD0+OUTMOD1+OUTMOD2;	// Set/Reset PWM mode
}

void Ports_config()
{
    P1OUT = CS + MISO + GDO0 + RST_BTN;   // CS up
    P1DIR = CS + P0 + MOSI + SCLK; 	// output on those pins
    P1REN = MISO + GDO0 + RST_BTN; 	// enable pull-up resistors
    P1SEL = SCLK + MISO + MOSI;    	// assign SPI pins to USI
    P1SEL2 = 0;                     // init PORT2
    P1IES = RST_BTN;				// falling edge interrupt
    P1IFG = 0;						// clear interrupt flags
    P2OUT = 0;                           
    P2DIR = LED + BUZZ;				// output on those pins
    P2SEL = BUZZ;					// connect buzzer to PWM port
    P2SEL2 = 0;
}

void WDT_config()
{
	WDTCTL = WDTPW + WDTTMSEL + WDTCNTCL + WDTSSEL
	  + WDTIS0 + WDTHOLD;			// clock WDT from ACLK, 1:8192 divisor
	IFG1 = 0;						// clear WDT interrupt flag
	IE1 = WDTIE;					// enable WDT interrupt
}

void Clock_config()					// configure basic clock module
{
  	DCOCTL = CALDCO_8MHZ;			// set 8 MHz clock
	BCSCTL1 = CALBC1_8MHZ;
	BCSCTL2 = DIVS1;				// SMCLK=DCO/4
	BCSCTL3 = LFXT1S1;				// select VLO as ACLK  
}

void SPI_config()					// configure USI module for SPI
{
	USICTL0 = USIPE7 + USIPE6 + USIPE5 + USIMST + USIGE + USIOE;
	USICKCTL = USISSEL1;			// use 1:1 SMCLK divider
}  

#pragma vector=PORT1_VECTOR
__interrupt void PORT1_ISR(void)	// PORT ISR routine
{
  	if (P1IFG & RST_BTN)
  	{
    		P2OUT_bit.LED = 0;		// power off LED (if on)
    		P1IFG_bit.RST_BTN = 0; 	// clear Reset button flag
			recFlg = 0;				// indicate no packet
  	}
  	if (P1IFG & GDO0)
  	{
    		recFlg = 1;   			// packet arrived
    		P1IFG_bit.GDO0 = 0;     // clear Radio Flag
			P1IE_bit.GDO0 = 0;		// disable GDO0 pin interrupt
  	}
	__low_power_mode_off_on_exit();
}

#pragma vector=USI_VECTOR			// USI SPI ISR routine
__interrupt void USI_ISR(void)
{
  	if (dataCnt)
	{
	 	USISRL = *dataPtr;			// load data to send to SPI shift reg
		USICNT = 8;					// transmit 8 bits 
		dataPtr++;
		dataCnt--;
	}  
	else if (dataRxCnt)
	{	  	
		*dataRxPtr = USISRL;		// get received data
		dataRxPtr++;				// update the pointer
		dataRxCnt--;
		if (dataRxCnt) 
		{  
			USISRL = 0;				// initiate new byte transfer
			USICNT = 8;
		}	
		else
		{
		 	USICTL1_bit.USIIE = 0;	// disable USI interrupt   
			__low_power_mode_off_on_exit(); 
		}  
	}
	else
	{  
		USICTL1_bit.USIIE = 0;		// disable USI interrupt   
		__low_power_mode_off_on_exit();
	}	
}

#pragma vector=TIMER0_A1_VECTOR		// Timer0 ISR routine
__interrupt void Timer0_ISR(void)
{
	TA0CTL_bit.TAIFG = 0;			// clear TimerA interrupt
	__low_power_mode_off_on_exit();
}  

#pragma vector=WDT_VECTOR			// WDT ISR routine
__interrupt void WDT_ISR(void)
{
 	IFG1 = 0;						// clear WDT interrupt flag
	__low_power_mode_off_on_exit();
}